<!doctype html>
<html>
  <head>
    <title>{{ metadata.title }}</title>
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <meta charset="UTF-8" />
    <link
      href="{{ metadata.style }}"
      rel="stylesheet"
      type="text/css"
      media="all"
    />
    <link
      href="/static/pygments.css"
      rel="stylesheet"
      type="text/css"
      media="all"
    />
  </head>
  <body>
    <div id="container">
    {% if header %}
      <header id="header">
      <center>
      {{ header | safe }}
      </center>
      <hr>
      </header>
    {% endif %}
        {% block content %}{% endblock %}
    {% if footer %}
      <footer id="footer">
      <hr>
      {{ footer | safe }}
      </footer>
    {% endif %}
    </div>
  </body>
</html>

