<div class="post-nav">
  <center>
    <span class="symbol">&lt;</span>{% if newer %}<a href="{{ newer }}">newr</a>{%
    else %}<span class="placeholder">null</span>{% endif %}<span
      class="symbol"
    ><span class="bar"></span></span>{% if older %}<a href="{{ older }}"
    >oldr</a>{% else %}<span class="placeholder">null</span>{% endif %}<span
      class="symbol"
    >&gt;</span>
  </center>
</div>
